function [Hmat,fvec,Amat,bvec,x,dx,row] = ComputeMatrix(X,y,KNN,kernel,grid_point,X_tilde)


n = size(y,1);
% size returns a vector of length 2 with height and width of x
d = size(X,2); 
% Reassign d with the value of d(2) makes d a scalar with the value indicating the 
% number of input variables


%Equally spaced grid.
Num_grid = round(grid_point^(1/d));

for i = 1:d,
    x(:,i) = linspace(min(X(:,i))', max(X(:,i))', Num_grid);
    %Calculate distance of equally space grid
    dx(i) = (x(2,i)-x(1,i))*1.1;
end


if d == 2,
    [x1,x2]=ndgrid(x(:,1),x(:,2));
    x=[x1(:),x2(:)];
elseif d == 3,
    [x1,x2,x3]=ndgrid(x(:,1),x(:,2),x(:,3));
    x=[x1(:),x2(:),x3(:)];  
elseif d == 4,
    [x1,x2,x3,x4]=ndgrid(x(:,1),x(:,2),x(:,3),x(:,4));
    x=[x1(:),x2(:),x3(:),x4(:)];  
elseif d == 5,
    [x1,x2,x3,x4,x5]=ndgrid(x(:,1),x(:,2),x(:,3),x(:,4),x(:,5));
    x=[x1(:),x2(:),x3(:),x4(:),x5(:)];  
end

x_tilde=(x-repmat(mean(x),size(x,1),1))./repmat(std(x),size(x,1),1);

%Small x is point for evaluation
%Capital X is observed points

l = zeros(length(x),d);

if kernel == 1
    %Gaussian kernel
    kerf = @(z) exp(-z.*z/2)/sqrt(2*pi);

elseif kernel == 2
    %Uniform kernel
    kerf = @(z) 1/2*(abs(z)<=1);

elseif kernel == 3
    %Epanechnikov kernel
    kerf = @(z) 3/4*(1-z^2)*(abs(z)<=1);
end



%Calculate distance of estimated points to 

for i = 1:length(x)
    for j = 1:n
        temp_pts = [x_tilde(i,:);X_tilde(j,:)];
        Dist(i,j) = pdist(temp_pts,'euclidean');
    end
end

%Get k-nearest bandwidth
for i = 1:length(x)
    SortDist = sort(Dist(i,:));
    R(i) = SortDist(KNN);
end
    
for i = 1:length(x),
    for j = 1:n,
        kernel(i,j) = kerf(Dist(i,j)/R(i));
    end
end





sum_prod_k = zeros(length(x),1);
sum_y_times_k=zeros(length(x),1);
sum_diff_X_times_yk=zeros(length(x),d);
sum_diff_X_times_k=zeros(length(x),d);
sum_diff_Xsq_times_k=zeros(length(x),d);
sum_diff_X_times_diff_X=zeros(length(x),d,d);

for i = 1:length(x)
    for j = 1:n
        sum_prod_k(i) = sum_prod_k(i) + kernel(i,j);
        sum_y_times_k(i) = sum_y_times_k(i) + y(j)*kernel(i,j);
        for k = 1:d

            sum_diff_X_times_yk(i,k) = sum_diff_X_times_yk(i,k) + (X(j,k)-x(i,k))*y(j)*kernel(i,j);
            sum_diff_X_times_k(i,k) = sum_diff_X_times_k(i,k)+(X(j,k)-x(i,k))*kernel(i,j);
            sum_diff_Xsq_times_k(i,k) = sum_diff_Xsq_times_k(i,k)+(X(j,k)-x(i,k))^2*kernel(i,j);
            
            for l = 1:d
                sum_diff_X_times_diff_X(i,k,l) = sum_diff_X_times_diff_X(i,k,l)+(X(j,k)-x(i,k))*(X(j,l)-x(i,l))*kernel(i,j);
            end
        end
    end
end


Hmat = zeros(length(x)+length(x)*d);
for i = 1:length(x)
    for j = 0:d
        for k = 0:d
            if j==0 & k==0 
                Hmat(i,i) = sum_prod_k(i);
            elseif j==0 & k~=0 
                Hmat(i+j*length(x),i+k*length(x)) = sum_diff_X_times_k(i,k);
            elseif j~=0 & k==0
                Hmat(i+j*length(x),i+k*length(x)) = sum_diff_X_times_k(i,j);
            elseif j~=0 & k~=0 & j==k
                Hmat(i+j*length(x),i+k*length(x)) = sum_diff_Xsq_times_k(i,j);
            elseif j~=0 & k~=0 & j~=k
                Hmat(i+j*length(x),i+k*length(x)) = sum_diff_X_times_diff_X(i,j,k);            
            end
        end
    end
end

fvec_alpha = sum_y_times_k;
fvec_beta = reshape(sum_diff_X_times_yk,length(x)*d,1);
fvec = -[fvec_alpha;fvec_beta];

Amat_afriat = [];


%consider only neibors and violated points in the previous loop
row = 1;
for i = 1:length(x)
    for j = 1:length(x)
        if i~=j
            if abs(x(i,:) - x(j,:)) <= dx
                Amat_afriat(row,i) = -1;
                Amat_afriat(row,j) = 1;
                for k = 1:d
                    Amat_afriat(row,i + length(x)*k) = x(i,k)-x(j,k);
                end
                row = row + 1;
                %cnt_cnst=cnt_cnst+1;
            end
        end 
    end
end

Amat = Amat_afriat;

bvec=zeros(row-1,1);




end