function [MSE_obs,MSE_grd,t,cnt_dlt,flagKNN] = LLKwSCtest(sum_exp,num_var,n,sigma,kernel,grid_point,minx_unif,maxx_unif,selection,random,error_model)


%Function of data generation.
if num_var == 1
    fun_data = @(x) x.^sum_exp;                                             %univariate
elseif num_var == 2
    fun_data = @(x) x(:,1).^(sum_exp/2).*x(:,2).^(sum_exp/2);               %bivariate
elseif num_var == 3
    fun_data = @(x) x(:,1).^(sum_exp/3).*x(:,2).^(sum_exp/3).*x(:,3).^(sum_exp/3);
elseif num_var == 4
    fun_data = @(x) x(:,1).^(sum_exp/4).*x(:,2).^(sum_exp/4).*x(:,3).^(sum_exp/4).*x(:,4).^(sum_exp/4);
elseif num_var == 5
    fun_data = @(x) x(:,1).^(sum_exp/5).*x(:,2).^(sum_exp/5).*x(:,3).^(sum_exp/5).*x(:,4).^(sum_exp/5).*x(:,5).^(sum_exp/5);
end



%%Random sample generation
if random==0
    rng(19);
end
X=[unifrnd(minx_unif,maxx_unif,n,num_var)];
if error_model == 0
    y=fun_data(X)+normrnd(0,sigma,n,1);
elseif error_model == 1
    y=fun_data(X).*normrnd(1,sigma,n,1);
end
y_true_obs = fun_data(X);

%Standarize data
X_tilde=(X-repmat(mean(X),n,1))./repmat(std(X),n,1);



tic
%Optimal KNN
warning('off','all')
[KNN,flagKNN] = Opt_KNN(X,y,kernel,selection,X_tilde);
warning('on','all')




[Hmat,fvec,Amat,bvec,x,dx,row] = ComputeMatrix(X,y,KNN,kernel,grid_point,X_tilde);


v_pt=[];
result=0;   %result = 0: exists some violations
            %result = 1: no violation


            
while result==0

format short g
[alpha1,beta1,x,yhat,Amat,row] = ComputeLocalLinear(y,X,Hmat,fvec,Amat,bvec,x,v_pt,row);



%Check concavity for estimated points by Afriat inequalities

result=1;
cnt_violation = 0;

v_pt = [];
cnt = 0;
for i = 1:length(x),
    for j = 1:length(x),
        if(i ~=j)
            if yhat(i) - yhat(j) +0.01 < beta1(i,:)*(x(i,:) - x(j,:))';                
                result=0;
                cnt=cnt+1;
                cnt_violation=cnt_violation+1;
                v_pt(cnt,:)=[i,j];
            end
        end
    end
end

if result == 1
    fprintf('Satisfied all Afriat inequalities.\n');
else
    fprintf('Violated %d Afriat inequalities.\n',cnt_violation);
end

end

t=toc;

%%Graph output (2D/3D)
% if num_var == 1
% %univariate
%     fig = figure;
%     fun = @(x) min(alpha1 + beta1 .* x);
%     figure(1)
%     xline = (min(x):0.1:max(x));
%     yline = arrayfun(fun, xline);
%     yline_true = arrayfun(fun_data, xline);
%     %plot(X, y,'r^', x, yhat,'bo', xline, yline,'-');
%     plot(X, y,'r^', x, yhat,'bo');
%     xlabel('x');
%     ylabel('y');
%     legend({'y (observad)' 'y_h_a_t'},'location','NE');
%     title('Local Linear Kernel with Shape Constraints');
%     figure(2)
%     plot(xline, yline,'-',xline, yline_true,'-',X, y,'r^');
%     xlabel('x');
%     ylabel('y');
%     legend({'estimated function' 'observed function' 'Observed points'},'location','NE');
%     title('Local Linear Kernel with Shape Constraints');
%     %savefig(fig,'C:\Users\Daisuke\Dropbox\Directed Study\Nonparametric estimation method\MATLAB\Experiments\KKL\results\FIG%d.fig',experiment_number);
%     close(fig)
% %bivariate
% elseif num_var == 2
%     fig = figure;
%     xlin1 = linspace(min(x(:,1)),max(x(:,1)),100);
%     xlin2 = linspace(min(x(:,2)),max(x(:,2)),100);
%     [X1,X2]=meshgrid(xlin1,xlin2);
%     %fun = @(x1, x2) min(alpha1 + sum(beta1.*[neighbor(x1,x2,1)*ones(length(beta1),1),neighbor(x1,x2,2)*ones(length(beta1),1)],2));
%     fun = @(x1, x2) min(alpha1 + sum(beta1.*[x1*ones(length(beta1),1),x2*ones(length(beta1),1)],2));
%     %fun = @(x1,x2) neighbor(x1,x2,alpha1,beta1,x,dx);
%     Z=arrayfun(fun, X1, X2);
%     %f=scatteredInterpolant(x(:,1),x(:,2),yhat);
%     %Z=f(X1,X2);
%     mesh(X1,X2,Z);
%     axis tight; hold on
%     %plot3(x(:,1),x(:,2),yhat,'.','MarkerSize',15)
%     plot3(X(:,1),X(:,2),y,'.','MarkerSize',5)
%     title('Local Linear Kernel with Shape Constraints')
%     xlabel('x1')
%     ylabel('x2')
%     zlabel('yhat')
%     %savefig(fig,'C:\Users\Daisuke\Dropbox\Directed Study\Nonparametric estimation method\MATLAB\Experiments\KKL\results\FIG%d.fig',experiment_number);
%     close(fig)
% end



%Calculate MSE
fun_est = @(x) min(alpha1 + sum(beta1.*repmat(x,length(beta1),1),2));
for i = 1:n
    y_est_obs(i) = fun_est(X(i,:));
end
y_est_obs = y_est_obs';

y_est_grd = yhat;
y_true_grd = fun_data(x);

%delete rows which have very negative value
cnt_dlt=0;
for i = 1:length(y_est_grd)
    if y_est_grd(i-cnt_dlt)<-10 
        y_true_grd(i-cnt_dlt)=[];
        y_est_grd(i-cnt_dlt)=[];
        cnt_dlt = cnt_dlt+1;
    end
end

MSE_obs = mean((y_true_obs-y_est_obs).^2);
MSE_grd = mean((y_true_grd-y_est_grd).^2);

fprintf('MSE for observed points:     %d\n',MSE_obs);
fprintf('MSE for equally grid points: %d\n',MSE_grd);

end