function [K_opt,flag] = Opt_KNN(X,y,kernel,selection,X_tilde)


n = size(y,1);
% size returns a vector of length 2 with height and width of x
d = size(X,2); 
% Reassign d with the value of d(2) makes d a scalar with the value indicating the 
% number of input variables

%Small x is point for evaluation
%Capital X is observed points

l = zeros(n,d);

flag = 0;

if kernel == 1
    %Gaussian kernel
    kerf = @(z) exp(-z.*z/2)/sqrt(2*pi);

elseif kernel == 2
    %Uniform kernel
    kerf = @(z) 1/2*(abs(z)<=1);

elseif kernel == 3
    %Epanechnikov kernel
    kerf = @(z) 3/4*(1-z^2)*(abs(z)<=1);
end

%Calculate distance of estimated points to 

for i = 1:n
    for j = 1:n
        temp_pts = [X_tilde(i,:);X_tilde(j,:)];
        Dist(i,j) = pdist(temp_pts,'euclidean');
    end
end





   %function of General Cross Validation
    function [obj] = GCV(k)
        
        A = zeros(1+d,1+d);
        B = zeros(1+d,1);
        
        %Get k-nearest bandwidth
        for ii = 1:n
            SortDist = sort(Dist(ii,:));
            R(ii) = SortDist(k);
        end

        for ii = 1:n,
            for jj = 1:n,
                prod_k(ii,jj) = kerf(Dist(ii,jj)/R(ii));
            end
        end
        sum_weight = sum(prod_k,2);
        for ii = 1:n
            for jj = 1:n,
                M(ii,jj) = prod_k(ii,jj)/sum_weight(ii);       
            end
        end
        
        ghat = M*y;
        obj = (n^(-1)*sum((y-ghat).^2))/(1-n^(-1)*trace(M))^2;      
    end



    %function of Leave-one-out Cross Validation
    function [MSE] = LOOCV(k)
        
        A = zeros(1+d,1+d);
        B = zeros(1+d,1);
        W = zeros(n-1,n-1);
        
        %Get k-nearest bandwidth
        for ii = 1:n
            SortDist = sort(Dist(ii,:));
            R(ii) = SortDist(k);
        end

        for ii = 1:n,
            Xmat = [ones(n,1),X-repmat(X(ii,:),n,1)];
            Xmat(ii,:)=[];
            Yvec = y;
            Yvec(ii)=[];
            ind = 1;
            for jj = 1:n,
                if jj~=ii,
                    W(ind,ind) = kerf(Dist(ii,jj)/R(ii));
                    ind=ind+1;
                end
            end
            
            ghat(ii,:) = [1,zeros(1,d)]*(Xmat'*W*Xmat)^(-1)*Xmat'*W*Yvec;
        end  
%             for jj = 1:n,
%                 if jj~=ii,
%                     A = A + prod_k(jj)*[1,X(jj,:)-X(ii,:)]'*[1,X(jj,:)-X(ii,:)];
%                     B = B + prod_k(jj)*[1,X(jj,:)-X(ii,:)]'*y(jj);
%                 end
%             end

%             temp_est = inv(A)*B;
%             ghat(ii,:) = temp_est(1);
%         end
        MSE=0;
        cnt = 0;
        for ii = 1:n,
            if isnan(ghat(ii))
            else
                MSE = MSE + (y(ii) - ghat(ii))^2;
                cnt=cnt+1;
            end
        end
        MSE = MSE/cnt;
    end


if selection == 2,
    obj(1)=1000000;
    for k = 2:n,
        [obj(k)]=GCV(k);
    end
    
    [m,K_opt] = min(obj);
    K_opt
elseif selection == 3,
    K_opt = round(n*0.1)
    
elseif selection ==1,
    obj(1)=1000000;
    for k = 2:round(n/5),
       [obj(k)]=LOOCV(k); 
    end
    [m,K_opt] = min(obj);
    K_opt
    if K_opt == round(n/5)
        flag = 1;
    end
end


 

end