function [Hmat,fvec,Amat,bvec,x,dx,row] = ComputeMatrix(X,y,h,kernel,grid_point)


n = size(y,1);
% size returns a vector of length 2 with height and width of x
d = size(X,2); 
% Reassign d with the value of d(2) makes d a scalar with the value indicating the 
% number of input variables


%Equally spaced grid.
Num_grid = round(grid_point^(1/d));

for i = 1:d,
    x(:,i) = linspace(min(X(:,i))', max(X(:,i))', Num_grid);
    %Calculate distance of equally space grid
    dx(i) = (x(2,i)-x(1,i))*1.1;
end


if d == 2,
    [x1,x2]=ndgrid(x(:,1),x(:,2));
    x=[x1(:),x2(:)];
elseif d == 3,
    [x1,x2,x3]=ndgrid(x(:,1),x(:,2),x(:,3));
    x=[x1(:),x2(:),x3(:)];  
elseif d == 4,
    [x1,x2,x3,x4]=ndgrid(x(:,1),x(:,2),x(:,3),x(:,4));
    x=[x1(:),x2(:),x3(:),x4(:)];  
elseif d == 5,
    [x1,x2,x3,x4,x5]=ndgrid(x(:,1),x(:,2),x(:,3),x(:,4),x(:,5));
    x=[x1(:),x2(:),x3(:),x4(:),x5(:)];  
end

%Small x is point for evaluation
%Capital X is observed points

l = zeros(length(x),d);

if kernel == 1
    %Gaussian kernel
    kerf = @(z) exp(-z.*z/2)/sqrt(2*pi);

elseif kernel == 2
    %Uniform kernel
    kerf = @(z) 1/2*(abs(z)<=1);

elseif kernel == 3
    %Epanechnikov kernel
    kerf = @(z) 3/4*(1-z^2)*(abs(z)<=1);
end




%Calculate kernel weights before declaring objective function
prod_k = ones(length(x),n);

for i = 1:length(x),
    for j = 1:n,
        for k = 1:d,    %Daisuke: loop to calculate product kernel
            prod_k(i,j) = prod_k(i,j)*kerf((X(j,k)-x(i,k))/h(k));
        end
    end
end



cnt_obj = 0;
cnt_cnst = 0;
% cvx_begin quiet
%     %cvx_precision low
%     variable alpha1(length(x))
%     variable beta1(length(x),d)
%     %Calculate sum of squared residuals with kernel weights (Gaussian)
%     SSR_k = 0;
%     for i = 1:length(x),
%         for j = 1:n,
%             %prod_k = 1;
%             %for k = 1:d,    %Daisuke: loop to calculate product kernel
%             %    prod_k = prod_k*kerf((X(j,k)-x(i,k))/h(k));
%             %end
%             if prod_k(i,j) ~= 0             
%             
% %                 SSR_k = SSR_k + (y(j)-alpha1(i)-(X(j,:)-x(i,:))*beta1(i,:)')^2*prod_k(i,j);
%                     SSR_k = SSR_k +  (y(j)-alpha1(i)-(X(j,:)-x(i,:))*beta1(i,:)')^2*prod_k(i,j) - y(j)^2*prod_k(i,j);
%                 cnt_obj = cnt_obj + 1;
%             end
%         end
%     end
%     minimize(SSR_k)
%     subject to
% % The purpose of this line is to impose monotonicity by restricting beta1 to be non-negative    
%         l <= beta1
% % These two loops construct the Afriat inequalities
%         for i = 1:length(x),
%             for j = 1:length(x),
%                 %Afriat inequalities are only considered when the distance is smaller than bandwidth
%                 if(i ~=j)
%                     %%%previous version using bandwidth (if abs(x(i,:) - x(j,:)) <= hs)
%                     if abs(x(i,:) - x(j,:)) <= dx
%                         alpha1(i) - alpha1(j) >= beta1(i,:)*(x(i,:) - x(j,:))';
%                         cnt_cnst=cnt_cnst+1;
%                     end
%                 end
%             end
%         end
% % Afriat inequalities which violate in the previous iteration
% %     for i = 1:length(v_pt)
% %         alpha1(v_pt(i,1)) - alpha1(v_pt(i,2)) >= beta1(v_pt(i,1),:)*(x(v_pt(i,1),:) - x(v_pt(i,2),:))';
% %         cnt_cnst=cnt_cnst+1;
% %     end
%         
% % Uncommenting this code will lead to a CRS estimator        
% %        for i = 1:n,
% %            0 == x(i,:)*beta1(:,i) - yhat(i);
% %        end
% cvx_end 

% for k = 1:d
%     for i = 1:length(x)
%         for j = 1:n
%             diff_X(i,j,k) = X(j,k) - x(i,k);
%         end
%     end
% end
% 
% for k = 1:d
%     for i = 1:length(x)
%         for j = 1:n
%             diff_X_sq(i,j,k) = (X(j,k) - x(i,k))^2;
%         end
%     end
% end



sum_prod_k = zeros(length(x),1);
sum_y_times_k=zeros(length(x),1);
sum_diff_X_times_yk=zeros(length(x),d);
sum_diff_X_times_k=zeros(length(x),d);
sum_diff_Xsq_times_k=zeros(length(x),d);
sum_diff_X_times_diff_X=zeros(length(x),d,d);

for i = 1:length(x)
    for j = 1:n
            sum_prod_k(i) = sum_prod_k(i) + prod_k(i,j);
            sum_y_times_k(i) = sum_y_times_k(i) + y(j)*prod_k(i,j);
        for k = 1:d

            sum_diff_X_times_yk(i,k) = sum_diff_X_times_yk(i,k) + (X(j,k)-x(i,k))*y(j)*prod_k(i,j);
            sum_diff_X_times_k(i,k) = sum_diff_X_times_k(i,k)+(X(j,k)-x(i,k))*prod_k(i,j);
            sum_diff_Xsq_times_k(i,k) = sum_diff_Xsq_times_k(i,k)+(X(j,k)-x(i,k))^2*prod_k(i,j);
            for l = 1:d
                sum_diff_X_times_diff_X(i,k,l) = sum_diff_X_times_diff_X(i,k,l)+(X(j,k)-x(i,k))*(X(j,l)-x(i,l))*prod_k(i,j);
            end
        end
    end
end


Hmat = zeros(length(x)+length(x)*d);
for i = 1:length(x)
    for j = 0:d
        for k = 0:d
            if j==0 & k==0 
                Hmat(i,i) = sum_prod_k(i);
            elseif j==0 & k~=0 
                Hmat(i+j*length(x),i+k*length(x)) = sum_diff_X_times_k(i,k);
            elseif j~=0 & k==0
                Hmat(i+j*length(x),i+k*length(x)) = sum_diff_X_times_k(i,j);
            elseif j~=0 & k~=0 & j==k
                Hmat(i+j*length(x),i+k*length(x)) = sum_diff_Xsq_times_k(i,j);
            elseif j~=0 & k~=0 & j~=k
                Hmat(i+j*length(x),i+k*length(x)) = sum_diff_X_times_diff_X(i,j,k);
            end
        end
    end
end

fvec_alpha = sum_y_times_k;
fvec_beta = reshape(sum_diff_X_times_yk,length(x)*d,1);
fvec = -[fvec_alpha;fvec_beta];

Amat_afriat = [];


%consider only neibors and violated points in the previous loop
row = 1;
for i = 1:length(x)
    for j = 1:length(x)
        if i~=j
            if abs(x(i,:) - x(j,:)) <= dx
                Amat_afriat(row,i) = -1;
                Amat_afriat(row,j) = 1;
                for k = 1:d
                    Amat_afriat(row,i + length(x)*k) = x(i,k)-x(j,k);
                end
                row = row + 1;
                cnt_cnst=cnt_cnst+1;
            end
        end 
    end
end

Amat = Amat_afriat;

bvec=zeros(row-1,1);




end